/*
 * NBCT
 * Copyright (c) 2007-2009, Yoshitaka Kameya
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms,  with or without
 * modification,  are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions  in  binary  form  must  reproduce  the  above
 *    copyright notice,  this list of conditions  and  the following
 *    disclaimer  in  the  documentation  and/or   other   materials
 *    provided with the distribution.
 *  * Neither  the  name  of  Tokyo Institute of Technology  nor the
 *    names of its contributors may be used to  endorse  or  promote
 *    products derived from  this  software  without  specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS"  AND  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO,  THE IMPLIED WARRANTIES OF MERCHANTABILITY  AND FITNESS
 * FOR  A  PARTICULAR  PURPOSE  ARE  DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL,    SPECIAL,   EXEMPLARY,   OR   CONSEQUENTIAL   DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT  LIABILITY,  OR  TORT  (INCLUDING  NEGLIGENCE  OR OTHERWISE)
 * ARISING  IN  ANY  WAY  OUT  OF  THE  USE  OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "nbc_def.h"
#include "nbc_ext.h"

static int hash(char *s)
{
  unsigned int v;
  for (v=0; *s != '\0'; s++) {
    v = ((v << CHAR_BIT) + *s) % HASH_SIZE;
  }
  return (int)v;
}

void init_hashtable(HASHTABLE *tp)
{
  int i;

  tp->serial_id = 0;
  tp->table = (HASH_NODE_PTR *)xmalloc(sizeof(HASH_NODE_PTR)*HASH_SIZE);

  for (i=0; i<HASH_SIZE; i++) tp->table[i] = NULL;
}

void release_hashtable(HASHTABLE *tp)
{
  int i;
  HASH_NODE_PTR q,r;

  for (i=0; i<HASH_SIZE; i++) {
    for (q = tp->table[i]; q != NULL; q = r) {
      r = q->next;
      xfree(q->key);
      xfree(q);
    }
  }

  xfree(tp->table);
}

int register_hashtable(HASHTABLE *tp, char *entry, int check_only)
{
  int cmp;
  HASH_NODE_PTR *p, q;

  p = tp->table + hash(entry);

  while ((*p) != NULL) {
    if ((cmp = strcmp(entry, (*p)->key)) == 0)
      return (*p)->id;
    else
      p = &((*p)->next);
  }

  if (check_only) return -1;

  q = (struct HASH_NODE *)xmalloc(sizeof(struct HASH_NODE));
  q->key = alloc_str(strlen(entry)+1);

  strncpy(q->key,entry,strlen(entry));
  q->id = tp->serial_id;
  q->next = NULL;

  tp->serial_id++;
  *p = q;

  return q->id;
}

char **make_entry_list(HASHTABLE *tp)
{
  int i;
  HASH_NODE_PTR q;
  char **el;

  el = (char **)xmalloc(sizeof(char *)*tp->serial_id);

  for (i=0; i<HASH_SIZE; i++) {
    q = tp->table[i];
    while (q != NULL) {
      el[q->id] = q->key;
      q = q->next;
    }
  }

  return el;
}
